@@ -10,6 +10,7 @@ from gpstime import gpsnow
 
 from . import const
 from . import nds
+from .exceptions import UnknownChannelError
 
 
 logger = logging.getLogger('DATA ')
@@ -312,14 +313,15 @@ class DataStore(QtCore.QObject):
         self.remote_cmd('find_channels')
 
     def add_channel(self, channel):
-        if self.available_channels and channel not in self.available_channels:
-            error = f"Channel '{channel}' not found in channel list."
+        if self.available_channels:
+            if channel not in self.available_channels:
+                error = f"Unknown channel '{channel}'."
+                self.signal_channel_add.emit((channel, error))
+                raise UnknownChannelError(error)
+        elif list(nds.find_channels(channel).keys()) != [channel]:
+            error = f"Unknown channel '{channel}'."
             self.signal_channel_add.emit((channel, error))
-            raise ValueError(error)
-        elif const.CHANNEL_RE.match(channel) is None:
-            error = f"Invalid channel name: {channel}"
-            self.signal_channel_add.emit((channel, error))
-            raise ValueError(error)
+            raise UnknownChannelError(error)
         self._channels[channel] += 1
         assert self._channels[channel] > 0
         self.signal_channel_add.emit((channel, None))
