@@ -204,7 +204,10 @@ def find_channels(channel_glob=None):
         'channel_type_mask': CHANNEL_TYPE_MASK,
     }
     if os.getenv('NDSSERVER', '').lower() == 'fake':
-        return {}
+        if channel_glob in ['*']:
+            return {}
+        else:
+            return {channel_glob: None}
     else:
         func = nds2.find_channels
     logger.debug("find_channels(**{})".format(kwargs))
