@@ -48,7 +48,7 @@ if os.getenv('ANTIALIAS'):
 STATUS_STYLES = {
     "data": "background: rgba(0,100,0,100);",
     "msg": "background: rgba(0,0,100,100);",
-    "error": "background: rgba(255,0,0,255); color: black; font-weight: bold;",
+    "error": "background: rgba(255,0,0,255); color: white; font-weight: bold;",
 }
 
 
@@ -218,6 +218,10 @@ class NDScope(*load_ui('scope.ui')):
         self.trendSecMinThresh.setText(str(const.TREND_TRANS_THRESHOLD['sec/min']))
         self.trendSecMinThresh.returnPressed.connect(self._set_trend_secmin)
 
+        self.trendMeanVisibility.stateChanged.connect(self.trend_mean_visibility_state_changed_slot)
+        self.trendMinVisibility.stateChanged.connect(self.trend_min_visibility_state_changed_slot)
+        self.trendMaxVisibility.stateChanged.connect(self.trend_max_visibility_state_changed_slot)
+
         ##########
         # axis
 
@@ -227,7 +231,7 @@ class NDScope(*load_ui('scope.ui')):
         # trigger
 
         self.trigger = Trigger()
-        self.trigger.sigLevelChanged.connect(self._update_triggerLevel)
+        self.trigger.level_changed_signal.connect(self._update_triggerLevel)
         self.triggerLevel.setValidator(QtGui.QDoubleValidator())
         self.triggerLevel.returnPressed.connect(self._set_trigger_level)
         self.triggerResetLevel.clicked.connect(self.reset_trigger_level)
@@ -248,6 +252,7 @@ class NDScope(*load_ui('scope.ui')):
 
         self.fontSizeSelect.valueChanged.connect(self.set_font_size)
         self.colorModeButtonGroup.buttonClicked.connect(self.set_color_mode)
+        self.lineWidthSelect.valueChanged.connect(self.set_line_width)
 
         ##########
         # export
@@ -264,10 +269,10 @@ class NDScope(*load_ui('scope.ui')):
         self.controlBar.hide()
         self.controlExpandButton.setIcon(self.style().standardIcon(QStyle.SP_TitleBarShadeButton))
         self.controlExpandButton.setText('')
-        self.controlExpandButton.clicked.connect(self._controlExpand)
+        self.controlExpandButton.clicked.connect(self.control_expand)
         self.controlCollapseButton.setIcon(self.style().standardIcon(QStyle.SP_TitleBarUnshadeButton))
         self.controlCollapseButton.setText('')
-        self.controlCollapseButton.clicked.connect(self._controlCollapse)
+        self.controlCollapseButton.clicked.connect(self.control_collapse)
 
         self.startButton.clicked.connect(self.start)
         self.startButton2.clicked.connect(self.start)
@@ -371,6 +376,17 @@ class NDScope(*load_ui('scope.ui')):
         elif mode == 'dark':
             self.darkModeButton.setChecked(True)
 
+    def set_line_width(self, width):
+        """set line width for all plots
+
+        This overrides any individual trace settings.
+
+        """
+        assert isinstance(width, int)
+        for plot in self.plots:
+            for cc in plot.channels.values():
+                cc.set_params(width=width)
+
     ##########
 
     def _base_ui(self):
@@ -383,7 +399,6 @@ class NDScope(*load_ui('scope.ui')):
         self.resetRangeButton2.setEnabled(False)
         self.windowTab.setEnabled(False)
         self.rangeTab.setEnabled(False)
-        self.triggerTab.setEnabled(False)
         self.crosshairTab.setEnabled(False)
         self.axisTab.setEnabled(True)
         self.exportTab.setEnabled(False)
@@ -399,7 +414,6 @@ class NDScope(*load_ui('scope.ui')):
         self.resetRangeButton2.setEnabled(True)
         self.windowTab.setEnabled(True)
         self.rangeTab.setEnabled(True)
-        self.triggerTab.setEnabled(False)
         self.crosshairTab.setEnabled(True)
         self.axisTab.setEnabled(True)
         self.exportTab.setEnabled(True)
@@ -415,7 +429,6 @@ class NDScope(*load_ui('scope.ui')):
         self.resetRangeButton2.setEnabled(True)
         self.windowTab.setEnabled(True)
         self.rangeTab.setEnabled(False)
-        self.triggerTab.setEnabled(True)
         self.crosshairTab.setEnabled(False)
         self.axisTab.setEnabled(False)
         self.exportTab.setEnabled(False)
@@ -517,11 +530,11 @@ class NDScope(*load_ui('scope.ui')):
     ##########
     # controls box
 
-    def _controlExpand(self):
+    def control_expand(self):
         self.controlBarSmall.hide()
         self.controlBar.show()
 
-    def _controlCollapse(self):
+    def control_collapse(self):
         self.controlBar.hide()
         self.controlBarSmall.show()
 
@@ -550,7 +563,7 @@ class NDScope(*load_ui('scope.ui')):
             if plot == p:
                 return loc
 
-    def add_plot(self, channels=None, yrange=None, **kwargs):
+    def add_plot(self, channels=None, **kwargs):
         """Add plot to the scope
 
         If provided `channels` should be a list of channel:property
@@ -570,13 +583,18 @@ class NDScope(*load_ui('scope.ui')):
         plot.set_font(self.font())
         plot.set_color_mode(self._color_mode)
         plot.channel_select_request.connect(self.select_channels)
-        plot.channel_add_complete.connect(self._channel_add_complete)
+        plot.request_reset.connect(self.reset)
         plot.new_plot_request.connect(self._add_plot_handler)
         plot.remove_plot_request.connect(self.remove_plot)
         plot.t0_reset.connect(self.reset_t0_relative)
-        plot.t_cursors_enabled.connect(self._update_t_cursors)
         plot.t_cursor_moved.connect(self._update_t_cursor)
+        plot.log_mode_toggled.connect(self.crosshair.redraw)
+        plot.log_mode_toggled.connect(self.trigger.redraw)
 
+        # filter out non-layout arguments
+        log_scale = kwargs.pop('log-scale', False)
+        # FIXME: deprecate the old "yrange" argument in favor of "y-range"
+        y_range = kwargs.pop('y-range', kwargs.pop('yrange', None))
         # FIXME: where to add if row/col not specified?  need some
         # sort of layout policy
         self.plotLayout.addItem(
@@ -611,16 +629,11 @@ class NDScope(*load_ui('scope.ui')):
                 ckwargs = ckwargs or {}
                 plot.add_channel(name, **ckwargs)
 
-        if self.plot0.t_cursors:
-            plot.enable_t_cursors()
-            plot.t_cursors.set_values(*self.plot0.t_cursors.get_values())
-
-        # set y ranges
-        if yrange in [None, 'auto']:
-            plot.enableAutoRange(axis='y')
-        else:
-            plot.disableAutoRange(axis='y')
-            plot.setYRange(*yrange)
+        # set Y scale first because it necessarily turns on autoscale
+        if log_scale:
+            plot.set_log_mode(log_scale)
+        if y_range:
+            plot.set_y_range(y_range)
 
         return plot
 
@@ -660,6 +673,7 @@ class NDScope(*load_ui('scope.ui')):
         # make copy of list cause we'll be changing it
         for chan in list(plot.channels.keys()):
             plot.remove_channel(chan)
+        plot.log_mode_toggled.disconnect()
         self.plotLayout.removeItem(plot)
 
     def plots4chan(self, channel):
@@ -732,6 +746,7 @@ class NDScope(*load_ui('scope.ui')):
     def _channel_select_dialog_result(self, plot, channel_list):
         self._dialog_cleanup('select')
         plot.set_channel_objs(channel_list)
+        self.reset()
 
     def _channel_add_callback(self, recv):
         """callback for when channel added"""
@@ -742,17 +757,6 @@ class NDScope(*load_ui('scope.ui')):
         logger.info(f"added channel: {channel}")
         self._update_triggerSelect()
 
-    def _channel_add_complete(self):
-        """callback for when a set of channel adds is complete
-
-        This call is signaled when a channel is added via the plot
-        menu
-
-        """
-        # FIXME: this really needs to be a better condition
-        if len(self.data.channels) == 1:
-            self.reset()
-
     def _channel_remove_callback(self, recv):
         """callback for when channel removed from plot"""
         channel, error = recv
@@ -801,8 +805,26 @@ class NDScope(*load_ui('scope.ui')):
                 self.set_color_mode('light')
             else:
                 self.set_color_mode('dark')
-        if template.get('color-mode'):
-            self.set_color_mode(template.get('color-mode'))
+        color_mode = template.get('color-mode')
+        if color_mode:
+            self.set_color_mode(color_mode)
+        trend = template.get('trend', 'auto')
+        if trend == 'auto':
+            self.trendAutoSelect.click()
+        elif trend == 'raw':
+            self.trendRawSelect.click()
+        elif trend == 'sec':
+            self.trendSecSelect.click()
+        elif trend == 'min':
+            self.trendMinSelect.click()
+        trend_auto_raw_sec = template.get('trend-auto-raw-sec')
+        if trend_auto_raw_sec:
+            self.trendRawSecThresh.clear()
+            self.trendRawSecThresh.insert(str(trend_auto_raw_sec))
+        trend_auto_sec_min = template.get('trend-auto-sec-min')
+        if trend_auto_sec_min:
+            self.trendSecMinThresh.clear()
+            self.trendSecMinThresh.insert(str(trend_auto_sec_min))
 
         # clear *all* plots from the scope, including the base 0,0 plot
         logger.info("clearing plots...")
@@ -810,13 +832,13 @@ class NDScope(*load_ui('scope.ui')):
             self.remove_plot(p, _force=True)
 
         def add_plot(p):
-            y_cursors = p.get('y-cursors')
-            if 'y-cursors' in p:
-                del p['y-cursors']
+            t_cursors = p.pop('t-cursors', None)
+            y_cursors = p.pop('y-cursors', None)
             plot = self.add_plot(**p)
+            if t_cursors:
+                plot.enable_t_cursors().load(t_cursors)
             if y_cursors:
-                plot.enable_y_cursors()
-                plot.y_cursors.set_values(*y_cursors)
+                plot.enable_y_cursors().load(y_cursors)
 
         # add the (0, 0) plot first
         add_plot(plot0)
@@ -827,10 +849,13 @@ class NDScope(*load_ui('scope.ui')):
                 continue
             add_plot(plot)
 
+        line_width = template.get('line-width')
+        if line_width:
+            self.lineWidthSelect.setValue(line_width)
+
         t_cursors = template.get('t-cursors')
         if t_cursors:
-            self.enable_t_cursors()
-            self.set_t_cursor_values(*t_cursors)
+            self.load_t_cursors(t_cursors)
 
         time_axis_mode = template.get('time-axis-mode')
         if time_axis_mode:
@@ -893,6 +918,9 @@ class NDScope(*load_ui('scope.ui')):
             window = (-span, 0)
         self.set_window(*window)
         trend = self.preferred_trend()
+        # Sets the visibility of the curves in case the trend type has changed.
+        # See the comment in `update_all_curves_visibility` for the reason this is needed.
+        self.update_all_curves_visibility()
         if trend == 'min':
             self.status_error("Online minute trends not supported.  Try a smaller window.")
             return
@@ -911,6 +939,9 @@ class NDScope(*load_ui('scope.ui')):
     def _data_request(self, force=False):
         ltrend, lstart, lend = self.last_data
         trend = self.preferred_trend()
+        # Sets the visibility of the curves in case the trend type has changed.
+        # See the comment in `update_all_curves_visibility` for the reason this is needed.
+        self.update_all_curves_visibility()
         start, end = self.get_range()
         if not force \
            and trend == ltrend \
@@ -1033,7 +1064,6 @@ class NDScope(*load_ui('scope.ui')):
 
     # SLOT
     def _data_online_done(self, signal):
-        self._disable_trigger()
         if self.crosshairGroup.isChecked():
             self._enable_crosshair()
         self._reset_time_axis_mode()
@@ -1098,8 +1128,12 @@ class NDScope(*load_ui('scope.ui')):
             else:
                 self._set_t0(data.range[1])
 
-        for plot in self.plots:
-            plot.update(data, self.t0)
+        if online and self.trigger.active and not trigger:
+            # don't update the plot if there was no trigger
+            pass
+        else:
+            for plot in self.plots:
+                plot.update(data, self.t0)
 
         if trigger and self.trigger.single:
             self.stop()
@@ -1265,6 +1299,43 @@ class NDScope(*load_ui('scope.ui')):
     def _set_trend_secmin(self, *args):
         const.TREND_TRANS_THRESHOLD['sec/min'] = int(self.trendSecMinThresh.text())
 
+    def set_all_mean_curves_visibility(self, visible):
+        for plot in self.plots:
+            plot.set_all_mean_curves_visibility(visible)
+
+    def set_all_min_curves_visibility(self, visible):
+        for plot in self.plots:
+            plot.set_all_min_curves_visibility(visible)
+
+    def set_all_max_curves_visibility(self, visible):
+        for plot in self.plots:
+            plot.set_all_max_curves_visibility(visible)
+
+    def update_all_curves_visibility(self):
+        # This check for the preferred trend
+        # is necessary because both the 'raw' PlotDataItem
+        # and the 'mean' PlotDataItem are indexed by the same
+        # key ('y') in the `curves` dictionary in the
+        # NDScopePlotChannel class.
+        if self.preferred_trend() == 'raw':
+            self.set_all_mean_curves_visibility(True)
+        elif self.preferred_trend() in ['sec', 'min']:
+            self.set_all_mean_curves_visibility(self.trendMeanVisibility.checkState())
+            self.set_all_min_curves_visibility(self.trendMinVisibility.checkState())
+            self.set_all_max_curves_visibility(self.trendMaxVisibility.checkState())
+
+    def trend_mean_visibility_state_changed_slot(self, state):
+        if self.preferred_trend() in ['sec', 'min']:
+            self.set_all_mean_curves_visibility(state)
+
+    def trend_min_visibility_state_changed_slot(self, state):
+        if self.preferred_trend() in ['sec', 'min']:
+            self.set_all_min_curves_visibility(state)
+
+    def trend_max_visibility_state_changed_slot(self, state):
+        if self.preferred_trend() in ['sec', 'min']:
+            self.set_all_max_curves_visibility(state)
+
     ##########
     # AXIS
 
@@ -1330,6 +1401,8 @@ class NDScope(*load_ui('scope.ui')):
             if nplot:
                 nplot.addItem(self.trigger.line, ignoreBounds=True)
                 nplot.disableAutoRange(axis='y')
+            self.trigger.plot = nplot
+            self.trigger.redraw()
             return True
         else:
             return False
@@ -1370,7 +1443,7 @@ class NDScope(*load_ui('scope.ui')):
         if self.data['raw'] and chan in self.data['raw']:
             y = self.data['raw'][chan].data['raw']
             yn = y[np.where(np.invert(np.isnan(y)))[0]]
-            value = np.mean(yn)
+            value = max(np.mean(yn), 0.1*np.max(yn))
         else:
             value = 0
         self.set_trigger_level(value)
@@ -1395,8 +1468,8 @@ class NDScope(*load_ui('scope.ui')):
     def _disable_trigger(self):
         """disable trigger, base method"""
         self.set_trigger_channel(None)
-        span = self.get_span()
-        self.set_window(-span, 0)
+        # span = self.get_span()
+        # self.set_window(-span, 0)
         logger.info("trigger disabled")
 
     def disable_trigger(self):
@@ -1431,8 +1504,6 @@ class NDScope(*load_ui('scope.ui')):
 
     def _enable_crosshair(self):
         """enable crosshair, base method"""
-        for plot in self.plots:
-            plot.disableAutoRange(axis='y')
         self._connect_crosshair()
         self.graphView.scene().sigMouseClicked.connect(self._clicked_crosshair)
         self.graphView.setCursor(Qt.CrossCursor)
@@ -1482,7 +1553,7 @@ class NDScope(*load_ui('scope.ui')):
         for plot in self.plots:
             if plot.sceneBoundingRect().contains(pos):
                 break
-        self.crosshair.update(pos, plot, self.t0)
+        self.crosshair.update(plot, pos, self.t0)
 
     def _update_crosshair_entry(self, recv):
         t, utc, local, y = recv
@@ -1503,42 +1574,19 @@ class NDScope(*load_ui('scope.ui')):
     ##########
     # CURSORS
 
-    def enable_t_cursors(self):
-        """enable the time cursors"""
-        if self.plot0.t_cursors:
-            return
-        for plot in self.plots:
-            plot.enable_t_cursors()
-        # update the cursor locations from plot0, so they all get the
-        # same value
-        self.plot0.t_cursors.reset(self.plot0)
-        logger.info("T cursors enabled")
-
-    def disable_t_cursors(self):
-        """disable the time cursors"""
-        if not self.plot0.t_cursors:
-            return
-        for plot in self.plots:
-            plot.disable_t_cursors()
-        logger.info("T cursors disabled")
-
-    def _update_t_cursors(self, enable):
-        if enable:
-            self.enable_t_cursors()
-        else:
-            self.disable_t_cursors()
-
     def _update_t_cursor(self, indval):
         index, value = indval
         for plot in self.plots:
             plot.t_cursors.set_values(**{index.lower(): value})
 
     def get_t_cursor_values(self):
-        if self.plot0.t_cursors:
-            return self.plot0.t_cursors.get_values()
+        return self.plot0.t_cursors.get_values()
 
     def set_t_cursor_values(self, t1=None, t2=None):
-        self.plot0.t_cursors.set_values(t1, t2)
+        self.plot0.enable_t_cursors().set_values(t1, t2)
+
+    def load_t_cursors(self, cursors):
+        self.plot0.enable_t_cursors().load(cursors)
 
     ##########
     # STYLE
@@ -1573,18 +1621,18 @@ class NDScope(*load_ui('scope.ui')):
             tabspec = util.cells_to_tabspec(cells)
             plot_item.update(tabspec)
             if any(plot.vb.state['autoRange']):
-                plot_item['yrange'] = 'auto'
+                plot_item['y-range'] = 'auto'
             else:
                 # convert to float from numpy.float64
-                plot_item['yrange'] = list(map(float, plot.vb.viewRange()[1]))
-            if plot.y_cursors:
-                plot_item['y-cursors'] = plot.y_cursors.get_values()
+                plot_item['y-range'] = list(map(float, plot.vb.viewRange()[1]))
+            if plot.log_mode:
+                plot_item['log-scale'] = True
+            if plot.t_cursors.are_visible():
+                plot_item['t-cursors'] = plot.t_cursors.export()
+            if plot.y_cursors.are_visible():
+                plot_item['y-cursors'] = plot.y_cursors.export()
             plots.append(plot_item)
 
-        t_cursors = self.get_t_cursor_values()
-        if t_cursors:
-            template['t-cursors'] = t_cursors
-
         template['time-axis-mode'] = self.timeAxisModeButtonGroup.checkedButton().text()
 
         template['plots'] = plots
